from datetime import timedelta
from typing import Any, Optional

from fastapi import APIRouter, Body, Depends, HTTPException
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy.orm import Session
from datetime import datetime
from app import crud, models, schemas
from app.api import deps
from app.core import security
from app.core.config import settings
from app.core.security import get_password_hash
from app.utils import (
    generate_password_reset_token,
    send_reset_password_email,
    verify_password_reset_token,
)

from google.oauth2 import id_token
from google.auth.transport import requests

router = APIRouter()


@router.post("/login/access-token")#, response_model=schemas.Token)
def login_access_token(
    db: Session = Depends(deps.get_db), form_data: OAuth2PasswordRequestForm = Depends(), add_time_code: Optional[str] = None
) -> Any:
    """
    OAuth2 compatible token login, get an access token for future requests
    """
    user = crud.user.authenticate(
        db, email=form_data.username, password=form_data.password
    )
    if not user:
        raise HTTPException(status_code=400, detail="Incorrect email or password")
    elif not crud.user.is_active(user):
        raise HTTPException(status_code=400, detail="Inactive user")
    access_token_expires = timedelta(minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
    return_msg = {
        "access_token": security.create_access_token(
            user.id, expires_delta=access_token_expires
        ),
        "token_type": "bearer",
    }
    if add_time_code:
        available_ser_no = crud.serial_number.available(db, ser_no=add_time_code)
        print(available_ser_no)
        if available_ser_no:
            user_in = schemas.UserUpdate(available_time=user.available_time+available_ser_no.time)
            crud.user.update(db, db_obj=user, obj_in=user_in)
        
            ser_no_in = schemas.SerialNumberUpdate(code=available_ser_no.code, is_used=True, used_datetime=str(datetime.now()), owner_id=user.id)
            crud.serial_number.update(db, db_obj=available_ser_no, obj_in=ser_no_in)
            print(available_ser_no.time, type(available_ser_no.time))
            return_msg['time_added'] = available_ser_no.time
        else:
            return_msg['time_added'] = -1
    return return_msg 

@router.post("/login/google/access-token")#, response_model=schemas.Token)
def login_access_token(
    db: Session = Depends(deps.get_db), form_data: OAuth2PasswordRequestForm = Depends(), add_time_code: Optional[str] = None
) -> Any:
    """
    OAuth2 compatible token login, get an access token for future requests
    """
    user = crud.user.google_auth(
        db, email=form_data.username
    )
    if not user:
        user_in = schemas.UserCreate(email=form_data.username, password=form_data.password)
        user = crud.user.create(db, obj_in=user_in)
    elif not crud.user.is_active(user):
        raise HTTPException(status_code=400, detail="Inactive user")
    access_token_expires = timedelta(minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
    return_msg = {
        "access_token": security.create_access_token(
            user.id, expires_delta=access_token_expires
        ),
        "token_type": "bearer",
    }
    if add_time_code:
        available_ser_no = crud.serial_number.available(db, ser_no=add_time_code)
        print(available_ser_no)
        if available_ser_no:
            user_in = schemas.UserUpdate(available_time=user.available_time+available_ser_no.time)
            crud.user.update(db, db_obj=user, obj_in=user_in)
        
            ser_no_in = schemas.SerialNumberUpdate(code=available_ser_no.code, is_used=True, used_datetime=str(datetime.now()), owner_id=user.id)
            crud.serial_number.update(db, db_obj=available_ser_no, obj_in=ser_no_in)
            print(available_ser_no.time, type(available_ser_no.time))
            return_msg['time_added'] = available_ser_no.time
        else:
            return_msg['time_added'] = -1
    return return_msg 



@router.post("/login/test-token", response_model=schemas.User)
def test_token(current_user: models.User = Depends(deps.get_current_user)) -> Any:
    """
    Test access token
    """
    return current_user


@router.post("/password-recovery/{email}", response_model=schemas.Msg)
def recover_password(email: str, db: Session = Depends(deps.get_db)) -> Any:
    """
    Password Recovery
    """
    user = crud.user.get_by_email(db, email=email)

    if not user:
        raise HTTPException(
            status_code=404,
            detail="The user with this username does not exist in the system.",
        )
    password_reset_token = generate_password_reset_token(email=email)
    send_reset_password_email(
        email_to=user.email, email=email, token=password_reset_token
    )
    return {"msg": "Password recovery email sent"}


@router.post("/reset-password/", response_model=schemas.Msg)
def reset_password(
    token: str = Body(...),
    new_password: str = Body(...),
    db: Session = Depends(deps.get_db),
) -> Any:
    """
    Reset password
    """
    email = verify_password_reset_token(token)
    if not email:
        raise HTTPException(status_code=400, detail="Invalid token")
    user = crud.user.get_by_email(db, email=email)
    if not user:
        raise HTTPException(
            status_code=404,
            detail="The user with this username does not exist in the system.",
        )
    elif not crud.user.is_active(user):
        raise HTTPException(status_code=400, detail="Inactive user")
    hashed_password = get_password_hash(new_password)
    user.hashed_password = hashed_password
    db.add(user)
    db.commit()
    return {"msg": "Password updated successfully"}
